"""
Klasa reprezentujaca permutacje zbioru {0, 1, ..., n-1},
wewnetrznie uzywajaca listy. Przykladowo, lista [3, 1, 2, 0]
reprezentuje permutacje 0->3, 1->1, 2->2, 3->0.
"""


class Permutation:
    def __init__(self, lst):
        if sorted(lst) != list(range(len(lst))):
            raise ValueError(f'{lst} does not represent a permutation.')
        self.lst = list(lst)

    def __str__(self):
        return f'Permutation({self.lst})'

    def __mul__(self, other):  # * operator
        n = len(self)
        if n != len(other):
            raise ValueError("!")
        new_lst = [self.lst[other.lst[i]] for i in range(n)]
        return Permutation(new_lst)

    def __len__(self):
        return len(self.lst)

    def __invert__(self):  # ~
        n = len(self)
        new_lst = [0] * n
        for i in range(n):
            new_lst[self.lst[i]] = i
        return Permutation(new_lst)

    def __xor__(self, other):  # ^
        return other * self * ~other

    def __pow__(self, k):  # **
        if k < 0:
            return (~self) ** (-k)
        x = Permutation(list(range(len(self))))
        for _ in range(k):
            x = x * self
        return x

    def __eq__(self, other):  # ==
        if len(self) != len(other):
            raise ValueError(f"Permutations are not of equal size.")
        return self.lst == other.lst

    def __getitem__(self, item):  # []
        return self.lst[item]

    def __call__(self, item):  # ()
        return self.lst[item]



if __name__ == "__main__":
    ident = Permutation([0,1,2,3])
    print(ident)
    sigma1 = Permutation([1,0,2,3])
    print(sigma1)
    print(sigma1 * sigma1)  # __ mul__
    print(Permutation([1, 2, 3, 0]) ** 4 == ident)  # __pow__, __eq__
    print(Permutation([1, 2, 3, 0]) ^ Permutation([1, 0, 2, 3]))  # __xor__
    print(sigma1[2])  # __getitem__
    print(sigma1(2))  # __call__
