import unittest
from permutation import Permutation


class Test(unittest.TestCase):
    def setUp(self):
        self.s = Permutation([1, 2, 3, 4, 0])
        self.t = Permutation([1, 0, 3, 4, 2])
        self.trans = Permutation([1, 0, 2, 3, 4])
        self.ident = Permutation([0, 1, 2, 3, 4])

    def test_init(self):
        lst = [0, 1, 4, 3, 2]
        p = Permutation(lst)
        self.assertListEqual(p.lst, lst)
        with self.assertRaises(ValueError):
            p = Permutation([0, 1, 5])

    def test_eq(self):
        self.assertEqual(self.s, self.s)
        self.assertNotEqual(self.s, self.t)

    def test_mul(self):
        self.assertEqual(self.s * self.t, Permutation([2, 1, 4, 0, 3]))
        self.assertEqual(self.t * self.s, Permutation([0, 3, 4, 2, 1]))
        self.assertEqual(self.t * self.t, Permutation([0, 1, 4, 2, 3]))
        self.assertEqual(self.s * self.s, Permutation([2, 3, 4, 0, 1]))
        self.assertEqual(self.trans * self.trans, self.ident)
        with self.assertRaises(ValueError):
            t = self.t * Permutation([0,1,2])

    def test_inverse(self):
        self.assertEqual(~self.s, Permutation([4, 0, 1, 2, 3]))
        self.assertEqual(~self.t, Permutation([1, 0, 4, 2, 3]))
        self.assertEqual(~self.trans, self.trans)

    def test_conjugate(self):
        self.assertEqual(self.s ^ self.t, Permutation([3, 0, 1, 4, 2]))

    def test_pow(self):
        self.assertEqual(self.s ** 5, self.ident)
        self.assertEqual(self.s ** 6, self.s)
        self.assertEqual(self.t ** 6, self.ident)



if __name__ == "__main__":
    unittest.main()
